# coding: utf-8

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from thrift2pyi.main import main

if __name__ == "__main__":
    main()
